﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace TetraminoSolver
{
    class BacktrackSolver
    {

        public Board Solve( ProblemSet problem )
        {
            var part = new PartialSolution( problem );
            return Solve_Recurse( part );
        }

        public Board Solve_Recurse(PartialSolution part)
        {
            Point nextPoint = part.CurrentBoard.GetNextFreeSlot(null);

            // No free space? Board Filled!
            if(nextPoint.X < 0) return part.CurrentBoard;

            // No more tetraminos? No solution found.
            if(part.LastPlacedTetramino == part.Problem.Tetraminos.Length - 1) return null;

            // For each possible position
            while( nextPoint.X >= 0 )
            {
                // And each possible adjustment/rotation
                foreach( var adj in Enum.GetValues(typeof(Tetramino.Adjustment)).Cast<Tetramino.Adjustment>().ToList() )
                {
                    PartialSolution nextPart = (PartialSolution)part.Clone();
                    Board returnedBoard = null;

                    // This style of tetramino has rotational symmetry, and this adjustment would have already been tried.
                    // The tetramino may be thrown in a different direction, but the topleft-most point of the tetramino
                    // should be at a position that is occupied / already tried.
                    if(nextPart.Problem.Tetraminos[nextPart.LastPlacedTetramino+1].RotationalSymmetry[(int)adj])
                        continue;

                    int nextId = nextPart.LastPlacedTetramino+1;
                    if(nextPart.CurrentBoard.ApplyTetramino( nextPart.Problem.Tetraminos[nextId], adj, nextPoint, nextId ))
                    {
                        nextPart.LastPlacedTetramino++;
                        returnedBoard = Solve_Recurse(nextPart);
                    }

                    // A solution was found? Pass it up.
                    if (returnedBoard != null) return returnedBoard;
                }
                nextPoint = part.CurrentBoard.GetNextFreeSlot(nextPoint);
            }

            // No solution possible from the provided base.
            return null;
        }

    }
}
