﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace TetraminoSolver
{
    class Board : ICloneable
    {

        private int _Height;
        public int Height { get { return _Height; }}
        private int _Width;
        public int Width  { get { return _Width;  }}

        public int[,] Slots = null;

        public Board(int width, int height)
        {
            _Height = height;
            _Width  = width;
            Slots = new int[Width,Height];
            for(int y = 0; y < Height; y++)
            for(int x = 0; x < Width;  x++)
                Slots[x,y] = -1;
        }

        public object Clone()
        {
            Board o = (Board)this.MemberwiseClone();
            o.Slots = new int[ this.Slots.GetLength(0), this.Slots.GetLength(1) ];
            Array.Copy( this.Slots, o.Slots, this.Slots.Length );
            return o;
        }

        public Point GetNextFreeSlot( Nullable<Point> cur )
        {   
            Point curP = cur ?? new Point(-1,0);
            int x = curP.X;
            int y = curP.Y;

            while( y < Height )
            {
                x++;
                if (x >= Width) y++;
                x %= Width;
                if (y >= Height)    return new Point(-1,0);
                if (Slots[x,y] < 0) return new Point( x,y);
            }
            return new Point(-1,0);
        }

        public Boolean ApplyTetramino( Tetramino tet, Tetramino.Adjustment adj, Point pos, int ID )
        {
            bool[,] pattern = tet.GetPatternData(adj);

            int w = pattern.GetLength(0);
            int h = pattern.GetLength(1);
            int offset_x = (w-1)/2;
            int offset_y = (h-1)/2;

            for(int y = 0; y < h; y++)
            for(int x = 0; x < w; x++)
            {
                if(!pattern[x,y]) continue; 

                int t_x = pos.X + x - offset_x;
                int t_y = pos.Y + y - offset_y;

                if( t_x < 0 | t_x >= Width | t_y < 0 | t_y >= Height ) return false;
                if( Slots[t_x, t_y] >= 0 ) return false;
            }

            for(int y = 0; y < h; y++)
            for(int x = 0; x < h; x++)
            {
                if (!pattern[x,y]) continue;
                Slots[pos.X + x - offset_x, pos.Y + y - offset_y] = ID;
            }

            return true;
        }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            for(int y = 0; y < Height; y++)
            {
                for(int x = 0; x < Width;  x++)
                    sb.Append( (Slots[x,y] < 10 && Slots[x,y] >= 0 ? "0" : "") + Slots[x,y].ToString() + " " );
                sb.AppendLine();
            }
            return sb.ToString();
        }

    }
}
