﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TetraminoSolver
{
    class Tetramino
    {

        public bool[,] PatternData        = new bool[7,7];
        public bool[]  RotationalSymmetry = new bool[4];

        public Tetramino( Pattern pat )
        {
            int [,] tempPat = new int[7,7];
            switch (pat)
            {
                case Pattern.Line: 
                    tempPat = new int[7,7]
                        {
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 1, 1, 1, 1 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 }
                        };
                    RotationalSymmetry[(int)Adjustment.Rotate_180] = true;
                    RotationalSymmetry[(int)Adjustment.Rotate_CCW] = true;
                    break;
                case Pattern.Square: 
                    tempPat = new int[7,7]
                        {
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 1, 1, 0, 0 },
                            { 0, 0, 0, 1, 1, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 }
                        };
                    RotationalSymmetry[(int)Adjustment.Rotate_180] = true;
                    RotationalSymmetry[(int)Adjustment.Rotate_CCW] = true;
                    RotationalSymmetry[(int)Adjustment.Rotate_CW]  = true;
                    break;
                case Pattern.T: 
                    tempPat = new int[7,7]
                        {
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 1, 1, 1, 0 },
                            { 0, 0, 0, 0, 1, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 }
                        };
                    break;
                case Pattern.L_Left: 
                    tempPat = new int[7,7]
                        {
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 1, 0, 0, 0 },
                            { 0, 0, 0, 1, 0, 0, 0 },
                            { 0, 0, 1, 1, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 }
                        };
                    break;
                case Pattern.L_Right: 
                    tempPat = new int[7,7]
                        {
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 1, 0, 0, 0 },
                            { 0, 0, 0, 1, 0, 0, 0 },
                            { 0, 0, 0, 1, 1, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 }
                        };
                    break;
                case Pattern.S_Left: 
                    tempPat = new int[7,7]
                        {
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 1, 0, 0, 0 },
                            { 0, 0, 1, 1, 0, 0, 0 },
                            { 0, 0, 1, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 }
                        };
                    RotationalSymmetry[(int)Adjustment.Rotate_180] = true;
                    RotationalSymmetry[(int)Adjustment.Rotate_CCW] = true;
                    break;
                case Pattern.S_Right: 
                    tempPat = new int[7,7]
                        {
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 },
                            { 0, 0, 0, 1, 0, 0, 0 },
                            { 0, 0, 0, 1, 1, 0, 0 },
                            { 0, 0, 0, 0, 1, 0, 0 },
                            { 0, 0, 0, 0, 0, 0, 0 }
                        };
                    RotationalSymmetry[(int)Adjustment.Rotate_180] = true;
                    RotationalSymmetry[(int)Adjustment.Rotate_CCW] = true;
                    break;
                default: break;
            }

            for( int y = 0; y < 7; y++ )
            for( int x = 0; x < 7; x++ )
                PatternData[x,y] = tempPat[y,x] > 0;
        }

        // Int equivalent is conveniently the number of CW rotations required.
        public enum Adjustment
        {
            None       = 0,
            Rotate_CW  = 1,
            Rotate_CCW = 2,
            Rotate_180 = 3
        }

        public enum Pattern
        {
            Line    = 0,
            Square  = 1,
            T       = 2,
            L_Left  = 3,
            L_Right = 4,
            S_Left  = 5,
            S_Right = 6
        }


        public bool[,] GetPatternData( Adjustment adj )
        {
            int n = PatternData.GetLength(0);

            bool[,] patternData = new bool[ n, n ];
            Array.Copy( PatternData, patternData, PatternData.Length );

            for( int i = 0; i < (int)adj; i++ )
            {
                bool[,] patternTemp = new bool[ n, n ];

                for(int y = 0; y < n; y++)
                for(int x = 0; x < n; x++)
                    patternTemp[x,y] = patternData[n-y-1,x];

                patternData = patternTemp;
            }

            return patternData;
        }

    }
}
